import Foundation

// Model & JSON

struct Task {
    let name: String
    let completed: Bool
    let position: Int
    let project: String?
    
    static func create(name: String)(completed: Bool)(position: Int)(project: String?) -> Task {
        return Task(name: name, completed: completed, position: position, project: project)
    }
}

let json = ["name": "Do this", "completed": false, "position": 10, "project": "Foo"]

// JSON parsing operators

infix operator </ { associativity left }
infix operator </? { associativity left }

func </ <A, B, C>(decoding: (A -> B -> C, [String: AnyObject])?, key: String) -> (B -> C, [String: AnyObject])? {
    if let (f, json) = decoding, x = json[key] as? A {
        return (f(x), json)
    } else {
        return nil
    }
}

func </ <A, B>(decoding: (A -> B, [String: AnyObject])?, key: String) -> B? {
    if let (f, json) = decoding, x = json[key] as? A {
        return f(x)
    } else {
        return nil
    }
}

func </? <A, B, C>(decoding: (A? -> B -> C, [String: AnyObject])?, key: String) -> (B -> C, [String: AnyObject])? {
    if let (f, json) = decoding {
        return (f(json[key] as? A), json)
    } else {
        return nil
    }
}

func </? <A, B>(decoding: (A? -> B, [String: AnyObject])?, key: String) -> B? {
    if let (f, json) = decoding {
        return f(json[key] as? A)
    } else {
        return nil
    }
}

// Decoding

let task = (Task.create, json) </ "name" </ "completed" </ "position" </? "project"
